--===========================================================================--
--
-- SS30IDE Hard Disk interface for the SWTPC
--
--  John Kent 20th April 2002
--
-- ss30ide.vhd
-- uses XC95108 CPLD
-- This version of the IDE interface is designed for 6809 systems
-- with 16 byte per I/O slot mapping.
--
-- IO_IDE + $00 = Data register 16 bits
-- IO_IDE + $01 = error / precompensation
-- IO_IDE + $02 = sector count
-- IO_IDE + $03 = sector number
-- IO_IDE + $04 = Cylinder LSB
-- IO_IDE + $05 = Cylinder MSB
-- IO_IDE + $06 = Drive / Head
-- IO_IDE + $07 = Status / Command
--
-- IO_Slot + $00 - $07 = ICS0_N (Drive 0)
-- IO_Slot + $08 - $0F = ICS1_N (Drive 1)
--
-- version 0.1 - design wont fit in version 0.0
--               so try reducing the printer port registers
--               by combining control & status
-- version 0.2 - ok ... that didn't work
--               guess we'll have to pull out all the printer stuff
--
-- version 0.3 - Now try direct register mapping.
--               Note that offset $01 and $01 need a state sequencer
--               so that any accesses preceeded by an access of
--               location $00 or location $08 will cause a 16 bit access.
--  
-------------------------------------------------------------------------------
library ieee;
   use ieee.std_logic_1164.all;
   use IEEE.STD_LOGIC_UNSIGNED.ALL;
   use ieee.numeric_std.all;

entity ss30ide is
  port(
    -- SS30 bus interface signals
    IOCS_N : in    std_logic;
    RST_N  : in    std_logic;
    RW     : in    std_logic;
    E_N    : in    std_logic;
    SS30_D : inout std_logic_vector(7 downto 0);
    RS     : in    std_logic_vector(3 downto 0);
    IRQ_N  : out   std_logic;
    FIRQ_N : out   std_logic;
    -- IDE Bus interface
    CS16_N : in    std_logic;
    PDIAG  : in    std_logic;
    DASP_N : in    std_logic;
    IA     : out   std_logic_vector(2 downto 0);
    IORDY  : in    std_logic;
    IRST_N : out   std_logic;
    IWR_N  : out   std_logic;
    IRD_N  : out   std_logic;
    ICS0_N : out   std_logic;
    ICS1_N : out   std_logic;
    ID     : inout std_logic_vector(15 downto 0);
    IIRQ   : in    std_logic;
	 -- Printer Port staus inputs
    FEH_N  : in    std_logic;
    PAP    : in    std_logic;
    ONOF   : in    std_logic;
    BSY    : in    std_logic;
    ACK_N  : in    std_logic;
	 -- Printer data port
    PD     : inout std_logic_vector(7 downto 0);
	 -- Printer control signals
    DSL_N  : out   std_logic;
    INI_N  : out   std_logic;
    ALF_N  : out   std_logic;
    STB_N  : out   std_logic;
	 -- Switch inputs
    SW     : in    std_logic_vector(3 downto 0);
    CLK    : in    std_logic
  );
end ss30ide;

-------------------------------------------------------------------------------
-- Architecture for memio Controller Unit
-------------------------------------------------------------------------------
architecture ide of ss30ide is
  -----------------------------------------------------------------------------
  -- Signals
  -----------------------------------------------------------------------------
  --
  signal data_16     : std_logic;
  signal IIRQ_EN     : std_logic;  -- hard disk interrupt enable SW<0>
  signal IIRQ_MO     : std_logic;  -- IRQ/ FIRQ select SW<1>
  signal ID_wr       : std_logic;
  signal ID_rd       : std_logic;
  signal ID_hi_latch : std_logic_vector(7 downto 0);
  signal ID_lo_latch : std_logic_vector(7 downto 0);

begin
 
-----------------------------------------------------------------------
--
-- IDE Register Bank selsect
--
-----------------------------------------------------------------------
IDE_select: process( E_N, RST_N, RW, RS, IOCS_N, data_16, ID_rd, ID_wr )
begin

  ICS0_N  <= not( RST_N and not(IOCS_N) and not(RS(3)) );
  ICS1_N  <= not( RST_N and not(IOCS_N) and     RS(3)  );
  IRST_N  <= RST_N;
  -- read strobe on all loactions except $01 and a 16 bit access
  ID_rd   <= RST_N and not(IOCS_N) and not(E_N) and RW and 
             not( not(RS(2)) and not(RS(1)) and RS(0) and data_16 );
  -- ide write strobe on all locations except $00
  ID_wr   <= RST_N and not(IOCS_N) and not(E_N) and not(RW) and 
             not( not(RS(2)) and not(RS(1)) and not(RS(0)) );
  IRD_N   <= not ID_rd;
  IWR_N   <= not ID_wr;
end process;

-----------------------------------------------------------------------
--
-- IDE Read/Write Process
-- 
----------------------------------------------------------------------
IDE_IO: process( E_N, RST_N, RW, RS, IOCS_N, data_16, ID_hi_latch, ID_lo_latch )
begin
  if( IOCS_N = '0' ) then
    if( RW = '0' ) then   -- write
	   ss30_d <= "ZZZZZZZZ";
      if( RS(2 downto 0) = "000" ) then 
		    -- data port access
		    ID(7 downto 0)  <= "00000000";
			 ID(15 downto 8) <= ss30_d;
          if( E_N'event and E_N = '0' ) then -- look for falling E edge to latch data
              ID_hi_latch <= ss30_d;         -- on write, latch high byte
		    end if;
	 	elsif( RS(2 downto 0) = "001" ) then 
		  -- second byte
		  if( data_16 = '0' ) then
		    -- 8 bit write
		    ID(7 downto 0)  <= ss30_d;
			 ID(15 downto 8) <= "00000000";
		  else
          -- 16 bit write
  		    ID(7 downto 0)  <= ss30_d;
          ID(15 downto 8) <= ID_hi_latch;
		  end if; -- data 16
		else -- all other registers 8 bit
		  ID(7 downto 0)  <= ss30_d;
		  ID(15 downto 8) <= "00000000";
		end if;
	 else -- rw = '1' read
	   ID <= "ZZZZZZZZZZZZZZZZ";
      if( RS(2 downto 0) = "000" ) then 
		  -- data port access
		  ss30_d <= ID(15 downto 8);
        if( E_N'event and E_N = '0' ) then -- look for falling E edge to latch data
			 ID_lo_latch <= ID(7 downto 0); -- on read, latch the low byte
		  end if;
		elsif( RS(2 downto 0) = "001" ) then 
		  -- second byte
		  if( data_16 = '0' ) then
		    -- 8 bit read
		    ss30_d <= ID(7 downto 0);
		  else
          -- 16 bit read
			 ss30_d <= ID_lo_latch;
		  end if; -- data 16
		else
		  -- all other registers are single byte read/write
		  ss30_d <= ID(7 downto 0);
		end if;
	 end if; -- RW
  else
    ID     <= "ZZZZZZZZZZZZZZZZ";
	 ss30_d <= "ZZZZZZZZ";
  end if; -- Reset / Chip select
end process;


----------------------------------------------------------------------
--
-- Synchronous stuff
-- Control IDE address line and 8 bit / 16 bit access control
--
----------------------------------------------------------------------

sync_stuff: process( E_N, RST_N, RS, IOCS_N )
begin
  if( E_N'event and E_N='1' ) then -- look for rising E edge to latch data
    if( RST_N = '0' ) then
		  data_16 <= '0';
		  IA <= "000";
    elsif( IOCS_N = '0' ) then
      if( RS(2 downto 0) = "000" ) then    -- data port access
		  IA <= RS(2 downto 0);
	     data_16 <= '1';
		elsif( RS(2 downto 0) = "001" ) then -- second byte
		  if( data_16 = '0' ) then
		    IA <= RS(2 downto 0);            -- 8 bit access updata address latch
		  else
		    data_16 <= '0';                  -- 16 bit access leave addr lines
		  end if;
		else
		  -- all other registers are single byte read/write
		  IA <= RS(2 downto 0);
		  data_16 <= '0';
		end if;
    end if; -- Reset / Chip select
  end if; -- E Clock
end process;


--/////////////////////////////////////////////////////////////////////
--// SS30IDE IRQ  Control
--// John Kent  19th April 2002
--/////////////////////////////////////////////////////////////////////
--//
--//  Interrupt output steering.
--//
--//  IIRQ_EN = IDE IRQ enable  - Switch 0
--//  IIRQ_MO = 'b0 -> IDE IRQ out - switch 1
--//            'b1 -> IDE FIRQ out
--//  IIRQ = IDE Interrupt Request
--//
irq_out: process( E_N, IIRQ, IIRQ_EN, IIRQ_MO )
variable IRQ_IN  : std_logic;
variable FIRQ_IN : std_logic;
begin
  if( E_N'event and E_N='1' ) then
    IIRQ_EN <= sw(0);
	 IIRQ_MO <= sw(1);
    IRQ_IN  := (IIRQ_EN and IIRQ and (not IIRQ_MO));
    FIRQ_IN := (IIRQ_EN and IIRQ and      IIRQ_MO );

    -- Emulate open drain signal for SS-30 bus
    if (IRQ_IN = '1') then
         IRQ_N <= '0';
    else
         IRQ_N <= 'Z';
    end if;

    -- Emulate open drain signal for SS-30 bus
    if (FIRQ_IN = '1') then
         FIRQ_N <= '0';
    else
         FIRQ_N <= 'Z';
    end if;
  end if;
end process;

end ide; --===================== End of architecture =======================--

